@@ -13,82 +13,13 @@ use Text::Template;
 use HTML::Show;
 use Ref::Util;
 
-our $VERSION = '0.006';    # VERSION
+our $VERSION = '0.007';    # VERSION
 
-=encoding utf-8
-
-=head1 NAME
-
-Chart::Dygraphs - Generate html/javascript charts from perl data using javascript library Dygraphs
-
-=head1 SYNOPSYS
-
- use Chart::Dygraphs qw(show_plot);
- 
- my $data = [map {[$_, rand($_)]} 1..10 ];
- show_plot($data);
-
- use Chart::Dygraphs qw(show_plot);
- use DateTime;
- 
- my $start_date = DateTime->now(time_zone => 'UTC')->truncate(to => 'hour');
- my $time_series_data = [map {[$start_date->add(hours => 1)->clone(), rand($_)]} 1..1000];
- 
- show_plot($time_series_data);
-
-=head1 DESCRIPTION
-
-Generate html/javascript charts from perl data using javascript library Dygraphs. The result
-is html that you could see in your favourite browser.
-
-The interface is "sub" oriented, but the API is subject to changes.
-
-=head1 FUNCTIONS
-
-=cut
-
-=head2 render_full_html
-
-=head3 Parameters
-
-=over
-
-=item * data:
-
-Data to be represented. The format is the perl version of the data expected by Dygraphs: L<http://dygraphs.com/data.html>
-
-=item * options:
-
-Hashref with options for graph. The format is the perl version of the options expected by Dygraphs: L<http://dygraphs.com/options.html>
-Optional
-
-=item * render_html_options
-
-Hashref with options controlling html output. With this you can inject html, javascript or styles.
-
-Supported options:
-
-=over
-
-=item * pre_graph_html
-
-=item * post_graph_html
-
-=item * dygraphs_div_id
-
-=item * dygraphs_javascript_object_name
-
-=item * dygraphs_div_inline_style
-
-=back
-
-=back
-
-=cut
+# ABSTRACT: Generate html/javascript charts from perl data using javascript library Dygraphs
 
 sub render_full_html {
     my %params = validate( @_,
-                           {  data    => { type => SCALAR | ARRAYREF },
+                           {  data    => { type => SCALAR | ARRAYREF | OBJECT },
                               options => { type => HASHREF, default => { showRangeSelector => 1 } },
                               render_html_options => { type     => HASHREF,
                                                        optional => 1,
@@ -119,6 +50,18 @@ sub _process_data_and_options {
     my $data           = shift();
     my $options        = shift();
     my $json_formatter = JSON->new->utf8;
+    local *PDL::TO_JSON = sub { $_[0]->unpdl };
+    if ( Ref::Util::is_blessed_ref($data) ) {
+        my $adapter_name = 'Chart::Dygraphs::Adapter::' . ref $data;
+        eval {
+            load $adapter_name;
+            my $adapter = $adapter_name->new( data => $data );
+            $data = $adapter->series();
+        };
+        if ($@) {
+            warn 'Cannot load adapter: ' . $adapter_name . '. ' . $@;
+        }
+    }
     return join( ',', _transform_data($data), $json_formatter->encode($options) );
 }
 
@@ -176,28 +119,28 @@ END_HTML
 
 }
 
-=head2 show_plot
-
-Opens the plot in a browser locally
-
-=head3 Parameters
-
-Data to be represented. The format is the same as the parameter data in render_full_html
-
-=cut
-
 sub show_plot {
     my @data_to_plot = @_;
 
     my $rendered_cells = "";
     my $numeric_id     = 0;
     for my $data (@data_to_plot) {
-        $rendered_cells .= _render_cell( _process_data_and_options( $data, { showRangeSelector => 1 } ),
-                                         {  dygraphs_div_id                 => 'graphdiv' . $numeric_id,
-                                            dygraphs_javascript_object_name => 'g' . $numeric_id
-                                         },
-                                         'chart_' . $numeric_id++
-        );
+        if ( ref $data eq 'Chart::Dygraphs::Plot' ) {
+            $rendered_cells .= _render_cell( _process_data_and_options( $data->data, $data->options ),
+                                             {  dygraphs_div_id                 => 'graphdiv' . $numeric_id,
+                                                dygraphs_javascript_object_name => 'g' . $numeric_id
+                                             },
+                                             'chart_' . $numeric_id++
+            );
+
+        } else {
+            $rendered_cells .= _render_cell( _process_data_and_options( $data, { showRangeSelector => 1 } ),
+                                             {  dygraphs_div_id                 => 'graphdiv' . $numeric_id,
+                                                dygraphs_javascript_object_name => 'g' . $numeric_id
+                                             },
+                                             'chart_' . $numeric_id++
+            );
+        }
     }
     my $plots = _render_html_wrap($rendered_cells);
     HTML::Show::show($plots);
@@ -207,6 +150,95 @@ sub show_plot {
 
 __END__
 
+=pod
+
+=encoding utf-8
+
+=head1 NAME
+
+Chart::Dygraphs - Generate html/javascript charts from perl data using javascript library Dygraphs
+
+=head1 VERSION
+
+version 0.007
+
+=head1 SYNOPSIS
+
+ use Chart::Dygraphs qw(show_plot);
+ 
+ my $data = [map {[$_, rand($_)]} 1..10 ];
+ show_plot($data);
+
+ use Chart::Dygraphs qw(show_plot);
+ use DateTime;
+ 
+ my $start_date = DateTime->now(time_zone => 'UTC')->truncate(to => 'hour');
+ my $time_series_data = [map {[$start_date->add(hours => 1)->clone(), rand($_)]} 1..1000];
+ 
+ show_plot($time_series_data);
+
+=head1 DESCRIPTION
+
+Generate html/javascript charts from perl data using javascript library Dygraphs. The result
+is html that you could see in your favourite browser.
+
+Example screenshot of plot generated with examples/time_series.pl:
+
+=for HTML <p>
+<img src="https://raw.githubusercontent.com/pablrod/p5-Chart-Dygraphs/master/examples/time_series.png" alt="Random time series plotted with Dygraphs">
+</p>
+
+=for markdown ![Random time series plotted with Dygraphs](https://raw.githubusercontent.com/pablrod/p5-Chart-Dygraphs/master/examples/time_series.png)
+
+The API is subject to changes.
+
+=head1 FUNCTIONS
+
+=head2 render_full_html
+
+=head3 Parameters
+
+=over
+
+=item * data:
+
+Data to be represented. The format is the perl version of the data expected by Dygraphs: L<http://dygraphs.com/data.html>
+
+=item * options:
+
+Hashref with options for graph. The format is the perl version of the options expected by Dygraphs: L<http://dygraphs.com/options.html>
+Optional
+
+=item * render_html_options
+
+Hashref with options controlling html output. With this you can inject html, javascript or styles.
+
+Supported options:
+
+=over
+
+=item * pre_graph_html
+
+=item * post_graph_html
+
+=item * dygraphs_div_id
+
+=item * dygraphs_javascript_object_name
+
+=item * dygraphs_div_inline_style
+
+=back
+
+=back
+
+=head2 show_plot
+
+Opens the plot in a browser locally
+
+=head3 Parameters
+
+Data to be represented. The format is the same as the parameter data in render_full_html
+
 =head1 AUTHOR
 
 Pablo Rodríguez González
@@ -256,5 +288,15 @@ AND CONTRIBUTORS "AS IS' AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES . THE IMP
             EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
               .
 
-=cut
+=head1 AUTHOR
 
+Pablo Rodríguez González <pablo.rodriguez.gonzalez@gmail.com>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2016 by Pablo Rodríguez González.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=cut
